clear all
% =====================================================================
%   < System Wide Runs >
%       : in preparation for submission to JMCB
%    
%   - by DongIk Kang
%   - last update : 2022/03/21
% =====================================================================

% =====================================================================
% (0) Parameter Values
% =====================================================================
delta = 0.1;          
aH = 1.2;              
phi = 0.8;             
aL = phi*aH;         
dlimit = 1.05;  
B = 2.4;

% parameter of interest
BB = (2:0.05:3.5)';             % probability of shock
num = max(size(BB));

% distribution of returns (lognormal distribution)
mu = -0.08;                                        
sigma = 0.4;                                       
mean = exp(mu + sigma^2/2);                       
variance = (exp(sigma^2)-1)*exp(2*mu+sigma^2);    
F = @(x) logncdf(x,mu,sigma);
f = @(x) lognpdf(x,mu,sigma);

% Marginal return for investors
I = @(x) 20*exp(-4*x)+0.01;

% =====================================================================
% (1) Credit Freeze
% =====================================================================
RHOs = zeros(num,1);
CUTs = zeros(num,1);
RSTARs = zeros(num,1);
DFLOORHs = zeros(num,1);
DFLOORLs = zeros(num,1);
cond_1 = zeros(num,1);
cond_2 = zeros(num,1);

for i = 1:num
    B = BB(i);    
    
    % 1. Fire sales prices 
    rhosfind = @(x) delta*(1/x - 1) - I(B-x*aH);
    rho_s = fsolve(rhosfind ,1);
    RHOs(i) = rho_s;
    
    % 2. Liquidation cutoffs (condition 1)
    d0_s = (1-delta*rho_s*aH)/(1-delta);      % period 0 debt if fire sale is severe
    liq_cuts = d0_s/aH;
    CUTs(i) = liq_cuts;   
    cond_1(i) = rho_s < liq_cuts;
    
    % 3. Debt floor (condition 2)
    Rf = @(x) x.*f(x);
    Rfind = @(x) rho_s - quad(Rf,0,x);
    Rstar_s = fsolve(Rfind ,1);
    dfloorH_s = (d0_s-rho_s*aH)/(1-F(Rstar_s));
    dfloorL_s = (d0_s-rho_s*phi*aH)/(1-F(Rstar_s));
    RSTARs(i) = Rstar_s;
    DFLOORHs(i) = dfloorH_s;
    DFLOORLs(i) = dfloorL_s;
    cond_2(i) = dfloorH_s > (1-phi)*aH*Rstar_s;
    cond_f(i) = dfloorL_s > dlimit;
end


% =====================================================================
% (2) No Freeze
% =====================================================================
RHOm = zeros(num,1);
CUTm = zeros(num,1);
RSTARm = zeros(num,1);
DFLOORm =zeros(num,1);
cond_3 = zeros(num,1);
cond_4 = zeros(num,1);

for i = 1:num
    B = BB(i);    
    
    % 1. Fire sales prices 
    rhomfind = @(x) delta*(1/x - 1) - I(B-x*phi*aH);
    rho_m = fsolve(rhomfind ,1);
    RHOm(i) = rho_m;
    
    % 2. Liquidation cutoffs (condition 1)
    d0_m = 1;
    liq_cutm = d0_m/aH;
    CUTm(i) = liq_cutm;   
    cond_3(i) = rho_m >= liq_cutm;
    
    % 3. Debt floor (condition 2)
    Rf = @(x) x.*f(x);
    Rfind = @(x) rho_m - quad(Rf,0,x);
    Rstar_m = fsolve(Rfind ,1);
    dfloor_m = (d0_m-rho_m*aH)/(1-F(Rstar_m));
    RSTARm(i) = Rstar_m;
    DFLOORm(i) = dfloor_m;
    cond_4(i) = dfloor_m <= (1-phi)*aH*Rstar_m;
        
end

cond_s = min([cond_1 cond_2 cond_f'],[],2);
BB_s = BB(cond_s>0);   RHO_s = RHOs(cond_s>0);
cond_m = max([cond_3 cond_4],[],2);
BB_m = BB(cond_m>0);   RHO_m = RHOm(cond_m>0);

plot(BB_s,RHO_s,BB_m,RHO_m,'--','LineWidth',2)
xlabel('B');   ylabel('\rho');
legend('system wide runs','credit contraction','Location','southeast','FontSize',12)
axis([2 3.5 0.4 1]) 
saveas(gcf,'C:\Users\dikang\Desktop\research_v\System Wide Runs (most recent)\draft\draft 2021_09_28_JMCB_resubmit\Run_B_matlab','epsc')


